# -*- code: utf-8 -*-
"""
    Text used by GOTO to do UX.
"""


def print_text(text, **kwargs):
    print(text.format(**kwargs))


warning = dict(
    adding_existing_magicword="""

Ah hoy!

    - You already have a magic word named {magicword},
    it points to: {uri}

    If you want to override it use:
        goto update {magicword} {newuri}
    """,

    removing_nonexisting_magicword="""
Ah hoy!

    - Attempt to remove non-existing shortcut with name {magicword}

    Are you sure it exists in the project you are in now?
    Type:

        goto list

    to see all shortcuts in this project.
    """,

    magicword_does_not_exist="""
Ah hoy!

    - The Magic word {magicword} does not exist.

    Are you sure it exists in the project you are in now?
    Type:

        goto list

    to see all shortcuts in this project.
    """,

    no_magicword_named_code="""
Ah hoy!
    - No magicword named code.
    Create it with
        goto add code <path to folder>
    in order to be able to run goto subl.
    """,

    missing_uri="""
Ah hoy!
    - Remember, a shortcut has a name and a target uri (or path).

    Try again by adding another argument with where the shortcut
    should go to:

                goto add {magicword} <uri>
    """,

    missing_magicword_and_uri="""
Ah hoy!
    - Remember, a shortcut has a name and a target uri (or path).

    Try again by adding both a name and a uri/path to where the shortcut
    should go to:

                goto add <magicword> <uri>

    example:
        goto add website http://example.org
    """
)


error = dict(
    magic_could_not_be_saved="""
Oh noes...!

    - Something clogged up the magic, and the magicwords could not be saved.

    Computer says: {message}

    If you want us to fix it - please tell us about this on github here:

        https://github.com/technocake/goto/issues

    Feel free to create a new issue if one does not already exist.

    """
)



# code: utf-8
"""
Magic is stored here.
"""
import json
import codecs
import os

from . import text
from .text import print_text


class GotoMagic():
    """ Goto's Black box.

        Nah, not really, it is just the place where
        magic shortcuts are created, updated, removed and
        used.

        tl-dr: saving magic as json.
    """

    def __init__(self, jfile):
        """ Loads json from jfile """
        self.jfile = jfile
        self.magic = load_magic(jfile)

    def save(self):
        """ Saves the magic to jsonfile jfile """
        save_magic(self.jfile, self.magic)

    def add_shortcut(self, magicword, uri):
        """ Adds a magic shortcut if it does not exist yet.
            If it exists, it warns the user.
        """
        uri = parse_uri(uri)
        if magicword in self.magic.keys():
            print_text(
                text.warning["adding_existing_magicword"],
                magicword=magicword,
                uri=self.magic[magicword],
                newuri=uri
            )
        else:
            self.magic[magicword] = uri

    def update_shortcut(self, magicword, uri):
        """ Simply overwrites the content of the magicword """
        uri = parse_uri(uri)
        self.magic[magicword] = uri

    def remove_shortcut(self, magicword):
        """ Simply removes a shortcut """
        try:
            self.magic.pop(magicword)
        except KeyError:
            print_text(
                text.warning["removing_nonexisting_magicword"],
                magicword=magicword
            )

    def show_shortcut(self, magicword):
        """ shows a shortcut """
        try:
            print(self.magic[magicword])
        except KeyError:
            print_text(
                text.warning["magicword_does_not_exist"],
                magicword=magicword
            )

    def get_uri(self, magicword):
        """ returns the uri from the shortcut of name <magicword> """
        try:
            return self.magic[magicword]
        except KeyError:
            print_text(
                text.warning["magicword_does_not_exist"],
                magicword=magicword
            )
            return None

    def list_shortcuts(self, verbose=False):
        """ Lists all magicwords.
            Optionally may list verbosedly all uris too
            if verbose = True.
        """
        if verbose:
            for key in sorted(self.magic.keys()):
                value = self.magic[key]
                print("%16s --> %s" % (key, value))
        else:
            for key in sorted(self.magic.keys()):
                print(key)

    def __getitem__(self, key):
        """
            Makes it possible to do print(magic[magiword])
        """
        return self.magic[key]

    def __missing__(self, key):
        """
            ux when magicword is missing
        """
        print_text(
            text.warning["magicword_does_not_exist"],
            magicword=key
        )

    def __setitem__(self, key, value):
        """ sets a magicword. Brutal style """
        self.magic[key] = value

    def __delitem__(self, key):
        """ removes a magicword. Brutal style """
        self.magic.pop(key)

    def __len__(self):
        """ returns number of magicwords. """
        return len(self.magic.keys())


def parse_uri(raw_uri):
    ''' Main goal right now: distinguish filesystem paths
        and urls/uris.
        If no scheme is present, assume it is
        a filesystem path. And test for path existence.
        This embeds a rule that all web-urls must start with
        either http:// or https:// for goto to handle them
        properly. One might ponder if goto should magically
        understand that www.gotomagic.com (without scheme)
        is a http-uri.
        TODO: here one could test if the raw_uri
              could work as a valid http(s):// uri,
              in the case where a uri is entered
              with no scheme.
        To handle cases such as: `goto add .`
        if the raw_uri is a path, get the absolute
        path and store that.
    '''
    candidate = os.path.abspath(raw_uri)
    if os.path.exists(candidate):
        return candidate
    else:
        # it is not a file or directory
        # So it must be an uri.
        # But which kind?
        return raw_uri


def is_file(raw_uri):
    ''' checks if the file or folder exist and returns True if so '''
    candidate = os.path.abspath(raw_uri)
    return os.path.exists(candidate)


def load_magic(jfile):
    "Loads the magic shortcuts from a json file object"
    if os.path.isfile(jfile) and os.path.getsize(jfile) > 0:
        with codecs.open(jfile, 'r', encoding="utf-8") as f:
            magic = json.loads(f.read())
    else:
        magic = {}
    return magic


def save_magic(jfile, magic):
    "saves the magic shortcuts as json into a file"
    with codecs.open(jfile, 'w+', encoding='utf-8') as f:
        try:
            data = json.dumps(magic, sort_keys=True, indent=4, ensure_ascii=False)
            f.write(data)
        except Exception as e:
            print_text(
                text.error["magic_could_not_be_saved"],
                message=e.message
            )



#!/usr/bin/env python
# code: utf-8
'Goto - the magic project that takes you where you need to be, now.'
from __future__ import absolute_import

import sys
import codecs

from .gotomagic.handlers import *
from .gotomagic.magic import GotoMagic, is_file
from .gotomagic import text
from .gotomagic.text import print_text


# make sure we print in utf-8
try:
    if sys.stdout.encoding != 'utf-8':
        sys.stdout = codecs.getwriter('utf-8')(sys.stdout.buffer, 'strict')
except:
    pass # TODO: implement utf-8 encoding of py2.7


def usage():
    return """
    Goto - the magic traveler, how may I help you?

    Wondering how to change project?
        project help                  Consult my brother in command

    The basics
        goto <magicword>                        Go to shortcut
        goto add    <magicword> <url or path>   Add shortcut      
        goto update <magicword> <url or path>   Update shortcut
        goto rm     <magicword>                 Remove shortcut
        goto show   <magicword>                 Show url of shortcut
        goto list                               List all shortcuts  
        goto list -v                            With the urls printed

    Working with folders and files
        goto <magicword>              Goto will cd to a folder shortcut by default. 
        goto cd   <magicword>         cd in terminal
        goto open <magicword>         Open in finder/file explorer
        goto -o   <magicword>                                    

    If you add a shortcut to a folder, and name it "code"...
        goto add code <path to folder with code>
        
    ...this command will open folder with Sublime Text
        goto subl                                

    ...this command will open folder with Visual Studio Code Text
        goto vscode           
    """

def main():
    jfile = sys.argv[1]
    magic = GotoMagic(jfile)

    if len(sys.argv) == 2:
        print(usage())
        exit(0)

    if len(sys.argv) > 2:
        if sys.argv[2] in ['help', '-h', '/?', '--help']:
            print(usage())
            exit(0)

        if sys.argv[2] == 'add':
            try:
                magic.add_shortcut(sys.argv[3], sys.argv[4])
                magic.save()
                print('Added magic word %s' % sys.argv[3])
                exit(0)
            except IndexError:
                if len(sys.argv) > 3:
                    print_text(
                        text.warning["missing_uri"],
                        magicword=sys.argv[3]
                    )
                else:
                    print_text(
                        text.warning["missing_magicword_and_uri"]
                    )
                exit(1)

        if sys.argv[2] == 'update':
            magic.update_shortcut(sys.argv[3], sys.argv[4])
            magic.save()
            print('Updated magic word %s' % sys.argv[3])
            exit(0)

        if sys.argv[2] == 'rm':
            try:
                magic.remove_shortcut(sys.argv[3])
                magic.save()
                print('Removed magic word %s' % sys.argv[3])
                exit(0)
            except Exception:
                print('Failed to remove magic word %s' % sys.argv[3])
                exit(1)

        if sys.argv[2] == 'show':
            magic.show_shortcut(sys.argv[3])
            exit(0)

        if sys.argv[2] == 'copy':
            copy_to_clipboard(str(magic.get_uri(sys.argv[3])))
            exit(0)

        if sys.argv[2] == 'list':
            magic.list_shortcuts(verbose=('-v' in sys.argv))
            exit(0)

        if sys.argv[2] == 'subl':
            try:
                open_sublime(magic['code'])
            except KeyError:
                print(text.warning["no_magicword_named_code"])
            exit(0)

        if sys.argv[2] == 'vscode':
            try:
                open_vscode(magic['code'])
            except KeyError:
                print(text.warning["no_magicword_named_code"])
            exit(0)

        if '-o' in sys.argv or '--open' in sys.argv or 'open' in sys.argv:
            open_folder(magic.get_uri(sys.argv[3]))
            exit(0)

        if sys.argv[2] == 'cd':
            open_terminal(magic.get_uri(sys.argv[3]))
            exit(0)
        # default
        url = magic.get_uri(sys.argv[2])
        if url is not None:
            if is_file(url):
                open_folder(url)
            else:
                open_link(magic[sys.argv[2]])

if __name__ == '__main__':
    main()



# code: utf-8
"""
project: goto

handlers to open different kinds of magic words.
Examples could be files, urls, visualstudio-solutions.
"""
import subprocess
import webbrowser
import os
import sys


def parse_magic_word(current_project, word):
    """ parses words in format of either:
            word
        or
            project.word

        reduces to a pair of project and word.

        TODO: not beeing used currently.
    """
    project, word = word.split('.') if "." in word else current_project, word
    return (project, word)


def copy_to_clipboard(url):
    import pyperclip
    pyperclip.copy(url)


def open_sublime(code):
    "hack"
    subprocess.call('subl "%s"' % code, shell=True)

def open_vscode(code):
    "hack"
    subprocess.call('code "%s"' % code, shell=True)

def open_folder(folder):
    "opens folders"
    folder = os.path.expanduser(folder)
    if sys.platform in ['linux', 'linux2']:
        subprocess.call('xdg-open "%s"' % folder, shell=True)
    elif sys.platform in ['darwin']:
        subprocess.call('open "%s"' % folder, shell=True)
    elif sys.platform in ['win32']:
        subprocess.call('start "%s"' % folder, shell=True)


def open_link(url):
    "Opens a link. Might do more stuff later."
    webbrowser.open(url)


def open_terminal(path):
    " Opens a new terminal window and cd-s to the given path "
    path = os.path.expanduser(path)
    subprocess.call(""" osascript <<END
                        tell app "Terminal" to do script "cd %s"
                        END
                    """ % path, shell=True)
